@@ -1,6 +1,36 @@
 import archinstall, getpass, time
 
-# Unmount and close previous runs
+def perform_installation(device, boot_partition, language):
+	"""
+	Performs the installation steps on a block device.
+	Only requirement is that the block devices are
+	formatted and setup prior to entering this function.
+	"""
+	with archinstall.Installer(device, boot_partition=boot_partition, hostname=hostname) as installation:
+		if installation.minimal_installation():
+			installation.set_keyboard_language(language)
+			installation.add_bootloader()
+
+			if len(packages) and packages[0] != '':
+				installation.add_additional_packages(packages)
+
+			if len(profile.strip()):
+				installation.install_profile(profile)
+
+			for user, password in users.items():
+				sudo = False
+				if len(root_pw.strip()) == 0:
+					sudo = True
+
+				installation.user_create(user, password, sudo=sudo)
+
+			if root_pw:
+				installation.user_set_pw('root', root_pw)
+
+			if len(aur.strip()) and aur.strip().lower() != 'no':
+				installation.add_AUR_support()
+
+# Unmount and close previous runs (in case the installer is restarted)
 archinstall.sys_command(f'umount -R /mnt', surpress_errors=True)
 archinstall.sys_command(f'cryptsetup close /dev/mapper/luksloop', surpress_errors=True)
 
@@ -9,6 +39,10 @@ archinstall.sys_command(f'cryptsetup close /dev/mapper/luksloop', surpress_error
   Not until we're satisfied with what we want to install
   will we continue with the actual installation steps.
 """
+
+keyboard_language = archinstall.select_language(archinstall.list_keyboard_languages())
+archinstall.set_keyboard_language(keyboard_language)
+
 harddrive = archinstall.select_disk(archinstall.all_disks())
 while (disk_password := getpass.getpass(prompt='Enter disk encryption password (leave blank for no encryption): ')):
 	disk_password_verification = getpass.getpass(prompt='And one more time for verification: ')
@@ -16,10 +50,11 @@ while (disk_password := getpass.getpass(prompt='Enter disk encryption password (
 		archinstall.log(' * Passwords did not match * ', bg='black', fg='red')
 		continue
 	break
+
 hostname = input('Desired hostname for the installation: ')
 if len(hostname) == 0: hostname = 'ArchInstall'
 
-while (root_pw := getpass.getpass(prompt='Enter root password (leave blank for no password): ')):
+while (root_pw := getpass.getpass(prompt='Enter root password (leave blank to leave root disabled): ')):
 	root_pw_verification = getpass.getpass(prompt='And one more time for verification: ')
 	if root_pw != root_pw_verification:
 		archinstall.log(' * Passwords did not match * ', bg='black', fg='red')
@@ -27,9 +62,17 @@ while (root_pw := getpass.getpass(prompt='Enter root password (leave blank for n
 	break
 
 users = {}
+new_user_text = 'Any additional users to install (leave blank for no users): '
+if len(root_pw.strip()) == 0:
+	new_user_text = 'Create a super-user with sudo privileges: '
+
 while 1:
-	new_user = input('Any additional users to install (leave blank for no users): ')
-	if not len(new_user.strip()): break
+	new_user = input(new_user_text)
+	if len(new_user.strip()) == 0:
+		if len(root_pw.strip()) == 0:
+			archinstall.log(' * Since root is disabled, you need to create a least one (super) user!', bg='black', fg='red')
+			continue
+		break
 	new_user_passwd = getpass.getpass(prompt=f'Password for user {new_user}: ')
 	new_user_passwd_verify = getpass.getpass(prompt=f'Enter password again for verification: ')
 	if new_user_passwd != new_user_passwd_verify:
@@ -37,9 +80,10 @@ while 1:
 		continue
 
 	users[new_user] = new_user_passwd
+	break
 
 aur = input('Would you like AUR support? (leave blank for no): ')
-if len(aur.strip()):
+if len(aur.strip()) and aur.lower() != 'no':
 	archinstall.log(' - AUR support provided by yay (https://aur.archlinux.org/packages/yay/)', bg='black', fg='white')
 
 profile = input('Any particular profile you want to install: ')
@@ -60,26 +104,10 @@ time.sleep(1)
 print(f' ! Formatting {harddrive} in 1...')
 time.sleep(1)
 
-def perform_installation(device, boot_partition):
-	with archinstall.Installer(device, boot_partition=boot_partition, hostname=hostname) as installation:
-		if installation.minimal_installation():
-			installation.add_bootloader()
-
-			if len(packages) and packages[0] != '':
-				installation.add_additional_packages(packages)
-
-			if len(profile.strip()):
-				installation.install_profile(profile)
-
-			for user, password in users.items():
-				installation.user_create(user, password)
-
-			if root_pw:
-				installation.user_set_pw('root', root_pw)
-
-			if len(aur.strip()):
-				installation.add_AUR_support()
-
+"""
+	Setup the blockdevice, filesystem (and optionally encryption).
+	Once that's done, we'll hand over to perform_installation()
+"""
 with archinstall.Filesystem(harddrive, archinstall.GPT) as fs:
 	# Use partitioning helper to set up the disk partitions.
 	if disk_password:
@@ -93,10 +121,12 @@ with archinstall.Filesystem(harddrive, archinstall.GPT) as fs:
 
 	if disk_password:
 		# First encrypt and unlock, then format the desired partition inside the encrypted part.
+		# archinstall.luks2() encrypts the partition when entering the with context manager, and
+		# unlocks the drive so that it can be used as a normal block-device within archinstall.
 		with archinstall.luks2(harddrive.partition[1], 'luksloop', disk_password) as unlocked_device:
 			unlocked_device.format('btrfs')
 			
-			perform_installation(unlocked_device, harddrive.partition[0])
+			perform_installation(unlocked_device, harddrive.partition[0], keyboard_language)
 	else:
 		harddrive.partition[1].format('ext4')
-		perform_installation(harddrive.partition[1], harddrive.partition[0])
\ No newline at end of file
+		perform_installation(harddrive.partition[1], harddrive.partition[0], keyboard_language)
\ No newline at end of file
